import pytest
from django.conf import settings

def pytest_configure():
    settings.configure(
        SECRET_KEY='test-key',
        INSTALLED_APPS=[
            'django.contrib.admin',
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.messages',
            'django.contrib.staticfiles',
            'django_genc',
        ],
        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': ':memory:',
            }
        },
        ROOT_URLCONF='django_genc.urls',
        MIDDLEWARE=[
            'django.middleware.security.SecurityMiddleware',
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.middleware.common.CommonMiddleware',
            'django.middleware.csrf.CsrfViewMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            'django.contrib.messages.middleware.MessageMiddleware',
            'django.middleware.clickjacking.XFrameOptionsMiddleware',
        ],
        TEMPLATES=[
            {
                'BACKEND': 'django.template.backends.django.DjangoTemplates',
                'DIRS': [],
                'APP_DIRS': True,
                'OPTIONS': {
                    'context_processors': [
                        'django.template.context_processors.debug',
                        'django.template.context_processors.request',
                        'django.contrib.auth.context_processors.auth',
                        'django.contrib.messages.context_processors.messages',
                    ],
                },
            },
        ],
        STATIC_URL='/static/',
        STATIC_ROOT='static',
        MEDIA_URL='/media/',
        MEDIA_ROOT='media',
    ) 


from django.db import models
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _
import re

class CountryCodeManager(models.Manager):
    def get_by_code(self, code):
        """
        Get a country by either its 2-letter or 3-letter code.
        Args:
            code (str): A 2 or 3 letter country code
        Returns:
            CountryCode: The matching country code instance
        Raises:
            CountryCode.DoesNotExist: If no country matches the code
            ValidationError: If the code format is invalid
        """
        if not code:
            raise ValidationError("Code cannot be empty")
            
        code = code.upper().strip()
        
        if not re.match(r'^[A-Z]{2,3}$', code):
            raise ValidationError("Code must be 2 or 3 uppercase letters")
            
        try:
            if len(code) == 2:
                return self.get(iso_code=code)
            else:
                return self.get(iso_code_3=code)
        except self.model.DoesNotExist:
            raise self.model.DoesNotExist(f"No country found with code '{code}'")

class CountryCode(models.Model):
    """Model to store GENC country codes and their ISO equivalents."""
    genc_code = models.CharField(
        max_length=3,
        unique=True,
        help_text="Three-digit GENC country code"
    )
    genc_name = models.CharField(
        max_length=100,
        help_text="Official GENC name"
    )
    genc_status = models.CharField(
        max_length=20,
        null=True,
        blank=True,
        help_text="GENC status (Exception, Extension, etc.)"
    )
    iso_code = models.CharField(
        max_length=2,
        unique=False,
        null=True,
        blank=True,
        help_text="Two-digit ISO country code"
    )
    iso_code_3 = models.CharField(
        max_length=3,
        unique=False,
        null=True,
        blank=True,
        help_text="Three-digit ISO country code (if different from GENC)"
    )
    iso_code_numeric = models.CharField(
        max_length=3,
        unique=False,
        null=True,
        blank=True,
        help_text="Numeric ISO country code"
    )
    iso_name = models.CharField(
        max_length=100,
        unique=False,
        null=True,
        blank=True,
        help_text="Official ISO name"
    )
    comment = models.TextField(
        null=True,
        blank=True,
        help_text="Additional information about the country code"
    )

    objects = CountryCodeManager()

    class Meta:
        verbose_name = "Country Code"
        verbose_name_plural = "Country Codes"
        ordering = ['genc_name']

    def __str__(self):
        return f"{self.genc_name} ({self.genc_code})"

    def clean(self):
        """Validate unique constraints before saving."""
        if self.genc_code:
            qs = CountryCode.objects.filter(genc_code=self.genc_code)
            if self.pk:
                qs = qs.exclude(pk=self.pk)
            if qs.exists():
                raise ValidationError({'genc_code': 'A country with this GENC code already exists.'})
                
        if self.iso_code:
            qs = CountryCode.objects.filter(iso_code=self.iso_code)
            if self.pk:
                qs = qs.exclude(pk=self.pk)
            if qs.exists():
                raise ValidationError({'iso_code': 'A country with this ISO code already exists.'})
                
        if self.iso_code_3:
            qs = CountryCode.objects.filter(iso_code_3=self.iso_code_3)
            if self.pk:
                qs = qs.exclude(pk=self.pk)
            if qs.exists():
                raise ValidationError({'iso_code_3': 'A country with this ISO-3 code already exists.'})
                
    def save(self, *args, **kwargs):
        self.full_clean()
        super().save(*args, **kwargs)

class CountryField(models.CharField):
    """Custom field that can handle both 2-digit ISO, 3-digit ISO, and 3-digit GENC codes."""
    description = "A field that stores GENC country codes but accepts ISO codes (both 2 and 3 digit)"

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 3
        super().__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        del kwargs['max_length']
        return name, path, args, kwargs

    def to_python(self, value):
        if value is None:
            return value
        
        value = str(value).strip().upper()
        
        # If it's already a GENC code, return it
        if len(value) == 3:
            try:
                # First check if it's a GENC code
                country = CountryCode.objects.get(genc_code=value)
                return country.genc_code
            except CountryCode.DoesNotExist:
                # If not a GENC code, check if it's a 3-digit ISO code
                try:
                    country = CountryCode.objects.get(iso_code_3=value)
                    return country.genc_code
                except CountryCode.DoesNotExist:
                    pass
            
        # If it's a 2-digit ISO code, look up the GENC code
        if len(value) == 2:
            try:
                country = CountryCode.objects.get(iso_code=value)
                return country.genc_code
            except CountryCode.DoesNotExist:
                raise ValidationError(
                    _('Invalid ISO country code: %(value)s'),
                    params={'value': value},
                )
        
        raise ValidationError(
            _('Country code must be either 2 or 3 characters'),
            params={'value': value},
        )

    def get_prep_value(self, value):
        """Convert the value to a string for storage in the database."""
        if value is None:
            return value
        return str(value).upper()

    def from_db_value(self, value, expression, connection):
        """Convert the value from the database to a Python object."""
        if value is None:
            return value
        return str(value).upper()

    def value_to_string(self, obj):
        """Convert the value to a string for serialization."""
        value = self.value_from_object(obj)
        return self.get_prep_value(value) 


from django.test import TestCase
from django.db.migrations.executor import MigrationExecutor
from django.db import connection
from django_genc.models import CountryCode


class MigrationTest(TestCase):
    @property
    def app(self):
        return 'django_genc'

    def setUp(self):
        self.executor = MigrationExecutor(connection)
        self.executor.migrate([(self.app, '0001_initial')])

    def test_migration_creates_table(self):
        """Test that the migration creates the CountryCode table."""
        tables = connection.introspection.table_names()
        self.assertIn('django_genc_countrycode', tables)

    def test_migration_loads_data(self):
        """Test that the migration loads data from the CSV file."""
        # Check that at least some data was loaded
        self.assertTrue(CountryCode.objects.exists())

        # Check that a known country exists
        usa = CountryCode.objects.filter(genc_code='USA').first()
        self.assertIsNotNone(usa)
        self.assertEqual(usa.genc_name, 'UNITED STATES')
        self.assertEqual(usa.iso_code, 'US')
        self.assertEqual(usa.iso_code_3, 'USA')

    def test_migration_constraints(self):
        """Test that the migration applies all constraints."""
        # Check that GENC code is unique
        with self.assertRaises(Exception):
            CountryCode.objects.create(
                genc_code='USA',
                genc_name='Duplicate USA',
                iso_code='XX',
                iso_code_3='XXX'
            )

        # Check that ISO code is unique
        with self.assertRaises(Exception):
            CountryCode.objects.create(
                genc_code='XXX',
                genc_name='Duplicate ISO',
                iso_code='US',
                iso_code_3='XXX'
            )

        # Check that ISO-3 code is unique
        with self.assertRaises(Exception):
            CountryCode.objects.create(
                genc_code='XXX',
                genc_name='Duplicate ISO-3',
                iso_code='XX',
                iso_code_3='USA'
            ) 


# Generated by Django 5.2 on 2025-05-01 11:40

from django.db import migrations, models
from pathlib import Path
import csv
import os


def load_combined_data(apps, schema_editor):
    CountryCode = apps.get_model('django_genc', 'CountryCode')
    
    # Get the path to the data file
    data_dir = Path(__file__).parent.parent / 'data'
    csv_file = data_dir / 'genc_iso_combined.csv'
    
    if not os.path.exists(csv_file):
        print(f'Warning: Data file not found at {csv_file}')
        return
        
    with open(csv_file, 'r') as f:
        reader = csv.DictReader(f)
        
        for row in reader:
            # Skip entries with no GENC code
            if not row['genc_code']:
                continue
                
            # Convert empty strings to None for optional fields
            iso_code = row['iso_code'] if row['iso_code'] and row['iso_code'] != '-' else None
            iso_code_3 = row['iso_code_3'] if row['iso_code_3'] else None
            iso_code_numeric = row['iso_code_numeric'] if row['iso_code_numeric'] else None
            iso_name = row['iso_name'] if row['iso_name'] else None
            comment = row['comment'] if row['comment'] else None
            genc_status = row['genc_status'] if row['genc_status'] else None
            
            CountryCode.objects.create(
                genc_code=row['genc_code'],
                genc_name=row['genc_name'],
                genc_status=genc_status,
                iso_code=iso_code,
                iso_code_3=iso_code_3,
                iso_code_numeric=iso_code_numeric,
                iso_name=iso_name,
                comment=comment
            )


class Migration(migrations.Migration):
    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CountryCode',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('genc_code', models.CharField(max_length=3, unique=True, help_text='Three-digit GENC country code')),
                ('genc_name', models.CharField(max_length=100, help_text='Official GENC name')),
                ('genc_status', models.CharField(blank=True, max_length=20, null=True, help_text='GENC status (Exception, Extension, etc.)')),
                ('iso_code', models.CharField(blank=True, max_length=2, null=True, help_text='Two-digit ISO country code')),
                ('iso_code_3', models.CharField(blank=True, max_length=3, null=True, help_text='Three-digit ISO country code (if different from GENC)')),
                ('iso_code_numeric', models.CharField(blank=True, max_length=3, null=True, help_text='Numeric ISO country code')),
                ('iso_name', models.CharField(blank=True, max_length=100, null=True, help_text='Official ISO name')),
                ('comment', models.TextField(blank=True, null=True, help_text='Additional information about the country code')),
            ],
            options={
                'verbose_name': 'Country Code',
                'verbose_name_plural': 'Country Codes',
                'ordering': ['genc_name'],
            },
        ),
        migrations.RunPython(load_combined_data),
    ]



from django.test import TestCase, Client
from django.contrib.auth import get_user_model
from django.urls import reverse
from django_genc.models import CountryCode


class CountryCodeAdminTest(TestCase):
    def setUp(self):
        # Create a superuser
        self.user = get_user_model().objects.create_superuser(
            username='admin',
            email='admin@example.com',
            password='password'
        )
        self.client = Client()
        self.client.login(username='admin', password='password')

        # Create a test country
        self.country = CountryCode.objects.get(
            genc_code='USA'
        )

    def test_admin_list_view(self):
        """Test that the admin list view works and shows all fields."""
        url = reverse('admin:django_genc_countrycode_changelist')
        response = self.client.get(url, {'q': 'USA'})
        
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'USA')
        self.assertContains(response, 'UNITED STATES')
        self.assertContains(response, 'Exception')
        self.assertContains(response, 'US')

    def test_admin_detail_view(self):
        """Test that the admin detail view works and shows all fields."""
        url = reverse('admin:django_genc_countrycode_change', args=[self.country.id])
        response = self.client.get(url)
        
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'USA')
        self.assertContains(response, 'UNITED STATES')
        self.assertContains(response, 'Exception')
        self.assertContains(response, 'US')

    def test_admin_search(self):
        """Test that the admin search functionality works."""
        url = reverse('admin:django_genc_countrycode_changelist')
        
        # Search by GENC code
        response = self.client.get(url, {'q': 'USA'})
        self.assertContains(response, 'UNITED STATES')
        
        # Search by GENC name
        response = self.client.get(url, {'q': 'UNITED STATES'})
        self.assertContains(response, 'UNITED STATES')
        
        # Search by ISO code
        response = self.client.get(url, {'q': 'US'})
        self.assertContains(response, 'UNITED STATES')

    def test_admin_filter(self):
        """Test that the admin filters work."""
        url = reverse('admin:django_genc_countrycode_changelist')
        
        # Filter by GENC status
        response = self.client.get(url, {'genc_status': 'Exception', 'q': 'USA'})
        self.assertContains(response, 'UNITED STATES')
        
        # Filter by non-existent status
        response = self.client.get(url, {'genc_status': 'NonExistent'})
        self.assertNotContains(response, 'UNITED STATES') 


"""
GENC (Geopolitical Entities, Names, and Codes) Django app.

This app provides models and fields for working with GENC country codes,
including conversion between 2-digit ISO codes and 3-digit GENC codes.
"""

__version__ = '0.1.0'


from django.contrib import admin
from django_genc.models import CountryCode

@admin.register(CountryCode)
class CountryCodeAdmin(admin.ModelAdmin):
    list_display = ('genc_name', 'genc_code', 'genc_status', 'iso_code', 'iso_code_3')
    list_filter = ('genc_status',)
    search_fields = ('genc_name', 'genc_code', 'iso_code', 'iso_code_3', 'iso_name')
    ordering = ('genc_name',)
    
    fieldsets = (
        ('GENC Information', {
            'fields': ('genc_code', 'genc_name', 'genc_status')
        }),
        ('ISO Information', {
            'fields': ('iso_code', 'iso_code_3', 'iso_code_numeric', 'iso_name')
        }),
        ('Additional Information', {
            'fields': ('comment',)
        }),
    ) 


from django.test import TestCase
from django.core.exceptions import ValidationError
from django_genc.models import CountryCode


class CountryCodeModelTest(TestCase):
    def setUp(self):
        # Clear existing data
        CountryCode.objects.all().delete()
        
        # Create a test country code
        self.country = CountryCode.objects.create(
            genc_code='TST',
            genc_name='Test Country 1',
            genc_status='Standard',
            iso_code='T1',
            iso_code_3='TST',
            iso_code_numeric='001',
            iso_name='Test Country One',
            comment='Test country'
        )

    def test_country_code_creation(self):
        """Test that a country code can be created with all fields."""
        self.assertEqual(self.country.genc_code, 'TST')
        self.assertEqual(self.country.genc_name, 'Test Country 1')
        self.assertEqual(self.country.genc_status, 'Standard')
        self.assertEqual(self.country.iso_code, 'T1')
        self.assertEqual(self.country.iso_code_3, 'TST')
        self.assertEqual(self.country.iso_code_numeric, '001')
        self.assertEqual(self.country.iso_name, 'Test Country One')
        self.assertEqual(self.country.comment, 'Test country')

    def test_unique_constraints(self):
        """Test that unique constraints are enforced."""
        # Try to create another country with the same GENC code
        with self.assertRaises(ValidationError):
            CountryCode.objects.create(
                genc_code='TST',
                genc_name='Another Test Country',
                iso_code='XX',
                iso_code_3='XXX'
            )

        # Try to create another country with the same ISO code
        with self.assertRaises(ValidationError):
            CountryCode.objects.create(
                genc_code='TS2',
                genc_name='Another Country',
                iso_code='T1',
                iso_code_3='XXX'
            )

        # Try to create another country with the same ISO-3 code
        with self.assertRaises(ValidationError):
            CountryCode.objects.create(
                genc_code='TS2',
                genc_name='Another Country',
                iso_code='XX',
                iso_code_3='TST'
            )

    def test_optional_fields(self):
        """Test that optional fields can be null."""
        country = CountryCode.objects.create(
            genc_code='TS2',
            genc_name='Test Country 2',
            genc_status=None,
            iso_code=None,
            iso_code_3=None,
            iso_code_numeric=None,
            iso_name=None,
            comment=None
        )
        self.assertIsNone(country.genc_status)
        self.assertIsNone(country.iso_code)
        self.assertIsNone(country.iso_code_3)
        self.assertIsNone(country.iso_code_numeric)
        self.assertIsNone(country.iso_name)
        self.assertIsNone(country.comment)

    def test_string_representation(self):
        """Test the string representation of the model."""
        self.assertEqual(str(self.country), 'Test Country 1 (TST)')

    def test_ordering(self):
        """Test that countries are ordered by GENC name."""
        CountryCode.objects.create(
            genc_code='TS2',
            genc_name='Test Country 2',
            iso_code='T2',
            iso_code_3='TS2'
        )
        CountryCode.objects.create(
            genc_code='TS3',
            genc_name='Test Country 3',
            iso_code='T3',
            iso_code_3='TS3'
        )

        countries = list(CountryCode.objects.all())
        self.assertEqual(countries[0].genc_name, 'Test Country 1')
        self.assertEqual(countries[1].genc_name, 'Test Country 2')
        self.assertEqual(countries[2].genc_name, 'Test Country 3') 